/**************************************************************************//**
 * @item     CosyOS Config
 * @file     mcucfg_cmx.h
 * @brief    CMSIS Cortex-M Core Config File
 * @author   迟凯峰
 * @version  V3.3.3
 * @date     2023.07.24
 ******************************************************************************/

#ifndef __MCUCFG_CMX_H
#define __MCUCFG_CMX_H

/******************************************************************************
 *                             USER Definitions                               *
 ******************************************************************************/

          //*** <<< Use Configuration Wizard in Context Menu >>> ***//

///////////////////////////////////////////////////////////////////////////////
// <s> 标准头文件
// <i> 定义与您的MCU相匹配的标准头文件，您的c文件中可不必再包含此文件。
// <i> 示例一：stm32f10x.h
// <i> 示例二：stm32f4xx.h
#define MCUCFG_STANDARDHEAD             "stm32f10x.h"
///////////////////////////////////////////////////////////////////////////////
// <o> 系统时钟
// <i> 告知CosyOS您所配置的系统时钟，单位为MHZ。
#define MCUCFG_SYSCLK                   72
#if !MCUCFG_SYSCLK
#error 非法的设置值！
#endif
///////////////////////////////////////////////////////////////////////////////
// <o> 系统滴答时钟源
// <0=> 外部时钟 <1=> 内核时钟
// <i> 在此配置系统滴答时钟源，您无需再额外配置。
// <i> 如果您使用了外部晶振且系统时钟为8的整数倍，可配置系统滴答时钟源为外部时钟，以提高系统滴答周期的精度。
#define MCUCFG_STKCLKSOURCE             1
///////////////////////////////////////////////////////////////////////////////
// <o> 系统启动任务的任务栈大小
// <i> 最小值：
// <i> 不使用浮点寄存器：64；
// <i> 使用浮点寄存器：200。
#define __STACKSIZE_STARTER__           1024
#if __STACKSIZE_STARTER__ < __BASICSTACKSIZE__
#error 非法的设置值！
#endif

// <o> 系统空闲任务的任务栈大小
// <i> 最小值：
// <i> 不使用浮点寄存器：64；
// <i> 使用浮点寄存器：200。
#define __STACKSIZE_SYSIDLE__           64
#if __STACKSIZE_SYSIDLE__ < __BASICSTACKSIZE__
#error 非法的设置值！
#endif
///////////////////////////////////////////////////////////////////////////////
// <o> 中断服务栈深度
// <i> 此项参数取决于您在中断中调用异步服务的总次数及中断的频率。
// <i> 可开启中断服务栈监控，监控历史上的最大值，再适当增大，以确保其不会溢出。
#define MCUCFG_ISSDEPTH                 64
#if MCUCFG_ISSDEPTH < 2
#error 非法的设置值！
#endif
///////////////////////////////////////////////////////////////////////////////
// <q> 中断服务栈监控
// <i> 是否开启中断服务栈监控？
// <i> 开启后，系统将自动监控中断服务栈的深度，并通过任务管理器输出。输出格式为：历史最大值/设置值。
// <i> 建议在调试阶段开启此项功能，正式的产品可禁用以提高性能。
#define MCUCFG_SVSTACKMONITOR           1
#if SYSCFG_DEBUGGING == 0 && MCUCFG_SVSTACKMONITOR == 1
#error 未启用DEBUG接口，不能开启服务栈监控！
#endif
///////////////////////////////////////////////////////////////////////////////
// <q> 任务栈监控
// <i> 是否开启任务栈监控？
// <i> 建议在调试阶段开启此项功能，正式的产品可禁用以提高性能。
#define MCUCFG_TASKSTACKMONITOR         1
#if SYSCFG_DEBUGGING == 0 && MCUCFG_TASKSTACKMONITOR == 1
#error 未启用DEBUG接口，不能开启任务栈监控！
#endif
///////////////////////////////////////////////////////////////////////////////

                //*** <<< end of configuration section >>> ***//

/******************************************************************************
 *                        Compiler Related Definitions                        *
 ******************************************************************************/

#include MCUCFG_STANDARDHEAD
#if __FPU_PRESENT == __ENABLED__ && __FPU_USED == __ENABLED__
#define MCUCFG_HARDWAREFPU  __ENABLED__
#else
#define MCUCFG_HARDWAREFPU  __DISABLED__
#endif

/******************************************************************************
 *                               OS Definitions                               *
 ******************************************************************************/

/* Header */
#include <string.h>

/* Memory */
#define _SYS_MEM_
#define _CODE_MEM_
#define _CONST_MEM_
#define _STACK_MEM_
#define _MALLOC_MEM_
#define _XDATA_MEM_
#define _DEBUG_MEM_

/* Register */
#define _SYS_REG_

/* Typedef */
#ifndef s64
typedef signed long long int s64;
#endif
#ifndef u64
typedef unsigned long long int u64;
#endif
#ifndef bool
typedef u8   bool;
#endif
typedef bool tBIT;
typedef u32  tSP;
typedef u32  tStackSize;
typedef u32  tDM;
typedef u32  tPC;
typedef u32  tGRP;
typedef u32  tSysTick;
typedef u32  tFetion;

/* Extern */
extern tSP vMSP;
extern u32 vBASEPRI;
extern void *vISS[MCUCFG_ISSDEPTH];
register void **vISP __ASM("r11");
extern u32 * volatile vDM_PSP;
extern volatile u32 vISS_DepthMAX;
extern __ASM void __isp_push(void *p);
extern __ASM void __stk_pushpop(bool, void *);
extern void __isv_handler(void);

/* * */
#define MCUCFG_MCUARC         __ARM__
#define MCUCFG_PCLEN          4
#define __USING__
#define __C51USING__
#define __STK_ATTRIBUTE__
#define __PSV_ATTRIBUTE__
#define __251_REENTRANT__
#define MCUCFG_DIRMSGTYPE     1
#define __DM_PSP__            vDM_PSP = (u32 *)__get_PSP()
#define __DM_VAR__            tDM r0__, tDM r1__, tDM r2__, tDM r3__
#define __DM_VAL__            0, 0, 0, 0
#define __DM_SIZE__           ((&m0 - &m0_ - 1) * 4)
#if MCUCFG_HARDWAREFPU == __ENABLED__
#define __CALLER_PUSH_FPU__   (18 * 4) /* {s0-s15,FPSCR,UNKNOW} */
#define __CALLEE_PUSH_FPU__   (16 * 4) /* {s16-s31} */
#else
#define __CALLER_PUSH_FPU__   0
#define __CALLEE_PUSH_FPU__   0
#endif
#define __CALLER_PUSH_REG__   (8 * 4) /* {r0-r3,r12,r14(lr),r15(pc),xPSR} */
#define __CALLEE_PUSH_REG__   (7 * 4) /* {r4-r10} */
#define __CALLER_PUSH__       (__CALLER_PUSH_FPU__ + __CALLER_PUSH_REG__)
#define __CALLEE_PUSH__       (__CALLEE_PUSH_FPU__ + __CALLEE_PUSH_REG__)
#define __BASICSTACKSIZE__    (__CALLER_PUSH__ + __CALLEE_PUSH__)
#define __BSP_ALIGN__         __align(8)
#define __STACKSIZE_TASKMGR__   (__BASICSTACKSIZE__ * 2 + (MCUCFG_HARDWAREFPU == __ENABLED__ ? 0 : 24))
#define __STACKSIZE_DEBUGGER__  (__BASICSTACKSIZE__ * 2)

/* API */
#define mSysTick_Cycle        ((1UL * MCUCFG_SYSCLK * SYSCFG_STKCYCLE) / (MCUCFG_STKCLKSOURCE ? 1 : 8))
#if !mSysTick_Cycle || mSysTick_Cycle > 0x00FFFFFF
#error 系统滴答周期溢出！
#endif
#define mSysTick_InitValue    mSysTick_Cycle
#define mSysTick_Counter      SysTick->VAL
#define mSTK_Disable          SysTick->CTRL = (MCUCFG_STKCLKSOURCE ? 0x04 : 0x00) | 0x01
#define mSTK_Enable           SysTick->CTRL = (MCUCFG_STKCLKSOURCE ? 0x04 : 0x00) | 0x01 | 0x02
#define mPSV_Trigger          *(u32 *)0xE000ED04 = 0x10000000
#define mTaskNode_Head_       tSP PSP;
#define mTaskNode_Tail_       tSP PSBP;
#define mEnterCritical        __set_BASEPRI(vBASEPRI)
#define mExitCritical         __set_BASEPRI(0)
#define mDisableIRQ           __disable_irq()
#define mEnableIRQ            __enable_irq()
#define mSys_Idle             __WFI()

#define mSys_INIT	\
do{	\
	vISP = vISS;	\
	vMSP = __get_MSP() - 2 * __BASICSTACKSIZE__;	\
	__set_PSP(vMSP);	\
	__set_CONTROL(0x02 | (MCUCFG_HARDWAREFPU == __ENABLED__ ? 0x04 : 0x00));	\
	SysTick->LOAD = mSysTick_InitValue;	\
	mSTK_Disable;	\
	/* PendSV_Handler Priority */	\
	*(u8 *)0xE000ED22 = 0xFF;	\
	/* SysTick_Handler Priority */	\
	*(u8 *)0xE000ED23 = 0xFF;	\
	/* 栈8字节对齐 */	\
	*(u32 *)0xE000ED14 |= 0x0200;	\
	/* CP11|CP10 */	\
	MCUCFG_HARDWAREFPU == __ENABLED__ ? *(u32 *)0xE000ED88 |= (0x0F<<20) : OS_NOPx1;	\
	/* ASPEN|LSPEN */	\
	MCUCFG_HARDWAREFPU == __ENABLED__ ? *(u32 *)0xE000EF34 &= 0x3FFFFFFF : OS_NOPx1;	\
	/* vBASEPRI */	\
	vBASEPRI <<= ~(SCB->AIRCR >> 8) & 7;	\
	vBASEPRI--;	\
	vBASEPRI <<= 4;	\
	__set_BASEPRI(0);	\
	mSTK_Enable;	\
	mEnableIRQ;	\
}while(false)

#define mTaskmgr_Counting	\
	counter2 = (counter1 * 100 * (MCUCFG_STKCLKSOURCE ? 1 : 8)) / MCUCFG_SYSCLK / counter2

#define mSTK_Counting	\
do{	\
	if(tick_temp <= mSysTick_Counter) break;	\
	vSTK_Counter1 += tick_temp - mSysTick_Counter;	\
	vSTK_Counter2++;	\
}while(false)

#if SYSCFG_DEBUGGING == __ENABLED__
#define mScheduler_INIT	\
	tStackSize stacklen;	\
	vScheduling_f = false
#define mTaskStack_LEN  stacklen = vTASKING->PSBP - __get_PSP() + __CALLEE_PUSH__
#else
#define mScheduler_INIT vScheduling_f = false
#define mTaskStack_LEN  do{}while(false)
#endif

#define mTaskStack_INIT	\
do{	\
	node_news->PSBP = (tSP)node_news->BSP + node_news->stacksize;	\
	if(node_news->PSBP % 8)	\
	{	\
		node_news->PSBP /= 8;	\
		node_news->PSBP *= 8;	\
		node_news->stacksize = node_news->PSBP - (tSP)node_news->BSP;	\
	}	\
	node_news->PSP = node_news->PSBP;	\
	*(u32 *)(node_news->PSP - __CALLER_PUSH_FPU__ - 4) = 0x01000000;	\
	*(u32 *)(node_news->PSP - __CALLER_PUSH_FPU__ - 8) = (u32)vACTBUF->entry;	\
	node_news->PSP-= __BASICSTACKSIZE__;	\
}while(false)

#if MCUCFG_TASKSTACKMONITOR == __ENABLED__
#define mEvery_Monitor	\
do{	\
	if(vTASKING->state == __STOPPED_TSOF__)	\
	{	\
		vFault.overflow_taskstack = true;	\
	}	\
	else if(vTASKING->stacksize < stacklen)	\
	{	\
		vTASKING->state = __STOPPED_TSOF__;	\
		vFault.overflow_taskstack = true;	\
	}	\
}while(false)
#else
#define mEvery_Monitor do{}while(false)
#endif

#if SYSCFG_SAFERUNTIME == __ENABLED__
#define mPUSH_Monitor	\
do{	\
	push_f = true;	\
	if(vTASKING->TPL > node_news->TPL)	\
	{	\
		vTASKING->counter = 0;	\
	}	\
}while(false)
#else
#define mPUSH_Monitor  push_f = true
#endif

#define mUsedTime_END	\
do{	\
	if(usedtime[0])	\
	{	\
		usedtime[0]--;	\
		usedtime[1] += mSysTick_InitValue - counter;	\
	}	\
	else	\
	{	\
		if(usedtime[1] >= counter)	\
		{	\
			usedtime[1] -= counter;	\
		}	\
		else	\
		{	\
			usedtime[1] += mSysTick_InitValue - counter;	\
		}	\
	}	\
	vTASKING->usedtime[0] += usedtime[0];	\
	vTASKING->usedtime[0] += (vTASKING->usedtime[1] + usedtime[1]) / mSysTick_Cycle;	\
	vTASKING->usedtime[1]  = (vTASKING->usedtime[1] + usedtime[1]) % mSysTick_Cycle;	\
}while(false)

#define mUsedTime_INIT	\
do{	\
	usedtime[0] = 0;	\
	usedtime[1] = counter;	\
}while(false)

#define mTaskStack_PUSH do{}while(false)

#define mTaskStack_POP  __stk_pushpop(push_f, node_news)

#define mISV_Do(svid)	\
do{	\
	__isp_push(&isv_);	\
	mPSV_Trigger;	\
}while(false)

#define mISV	\
	if(vISP > vISS) __isv_handler()

#define miWriteFlagBits	\
	if(!isv_.value)	\
	{	\
		do{}while(false)

#endif
