
### 任务调度
CosyOS的任务调度类型包括抢占式调度、时间片轮转调度，不同优先级采用抢占式调度，相同优先级采用时间片轮转调度。<br>

CosyOS的任务调度时机分为定时调度与临时调度。<br>
* **定时调度** 在每个系统滴答周期，SysTick_Handler/定时器0中断都会触发一次任务调度。<br>
* **临时调度** <br>
   **1.**  当前任务被阻塞、挂起、超时、停止、删除时，会立即触发任务调度；<br>
   **2.**  恢复任务时，如果恢复后该任务可以运行并且它的优先级高于当前任务的优先级，会立即触发任务调度；<br>
   **3.**  设置任务优先级时，如果该任务可以运行并且它的新优先级高于当前任务的优先级，会立即触发任务调度。<br>
   **4.**  用户通过调用服务，直接触发任务调度，包括任务级任务调度和中断级任务调度。<br>

CosyOS的任务调度机制可最大程度的实现尽可能实时高效的任务切换，使任务的切换效率无与伦比。<br>

### 任务优先级分组
您可在系统配置文件中配置任务优先级分组。共有六个分组可供选择，8级、16级、32级、64级、128级、256级，您也可自定义级数。
如您配置的分组为8级，则最高优先级为7级，最低优先级为0级。<br>

### 时间片模式
CosyOS有三种时间片的定义模式，全局时间片、算法时间片、自定义时间片，需在系统配置文件中定义。<br>
 **1. 全局时间片**  所有任务采用统一的时间片。<br>
 **2. 算法时间片**  不同优先级的时间片不同，采用一个算法公式来描述。<br>
 **3. 自定义时间片** 不同优先级的时间片不同，用户自定义各优先级的时间片。<br>

时间片的取值范围：1~65535，单位为滴答周期。

### 安全运行时
安全运行时是CosyOS的安全关键技术之一，是为了防止某任务长期独占或超时使用处理器。当任务累计运行时间超过安全运行时，仍不能被比它优先级低的任务所切换，则进入超时状态并放弃处理器的使用权，直到系统空闲任务运行时才能恢复。<br>
安全运行时的取值范围：0~65535，单位为时间片，0为无限长。<br>
所有空闲任务无法应用安全运行时，创建任务时，安全运行时输入0即可。<br>
安全运行时的应用技巧：在开发初期，所有任务的安全运行时可设置为0，待开发测试阶段中后期，再根据各任务的特点和测试经验合理的调整为有效值。

### 软件定时器
CosyOS的软件定时器分为三类，延时定时器、定时中断定时器、定时查询定时器。
1. **延时定时器** 每个任务各有一个，用来在此任务中实现阻塞延时或超时计数。
2. **定时中断定时器** 每个定时中断任务/钩子绑定一个定时中断定时器，至多可以有64个，定时器ID从0开始，用户通过调用API进行定时操作后，定时器将自动开始计数。
3. **定时查询定时器** 每个定时查询任务/钩子绑定一个定时查询定时器，至多可以有64个，定时器ID从0开始，用户通过调用API进行定时操作后，定时器将自动开始计数。

### 支持任务/钩子
 **一、系统任务** <br>
由操作系统自主创建并自动启动的任务。<br>
 **1. 任务管理器（Taskmgr）**  优先级固定为最高级。<br>
 **2. 调试任务（Debugger）**  优先级固定为最高级，用做命令行和任务管理器的串口接收解析。<br>
 **3. 启动任务（Starter）**  优先级固定为比最高级低一级，通过调用启动钩子来启动用户任务，而后删除自身。<br>
 **4. 系统空闲任务（Sysidle）**  优先级固定为最低级0级，是系统级的空闲任务。<br>

 **二、系统钩子** <br>
由操作系统自主创建并自动调用的函数，并由用户来写代码。<br>
 **1. 初始化钩子（init_hook）**  在主函数中首先被调用，适用于初始化时钟、IO、寄存器等工作。<br>
 **2. 启动钩子（start_hook）**  在启动任务中被调用，用于启动用户任务。用户可在启动钩子内调用API“uStartTask”来启动用户任务。<br>
 **3. 滴答钩子（tick_hook）**  每个系统滴答周期，在SysTick_Handler中都会被调用一次，适用于每滴答周期/秒/分/时/...做一次的工作。<br>
 **4. 空闲钩子（idle_hook）**  在系统空闲任务中被调用，用户可添加自己的代码做一些没有实时性要求的工作。<br>

 **三、定时任务/钩子** <br>
包括定时中断任务/钩子和定时查询任务/钩子，其实质都是软件定时器中断。<br>
 **1. 定时中断任务/钩子**  由用户设定定时时间，当定时器溢出时，系统将自动恢复/调用与其绑定的任务/钩子并自动重复定时（如果您已开启定时器的自动重装载）。<br>
 **2. 定时查询任务/钩子**  由用户设定定时时间，当定时器溢出后，系统在每个滴答周期都会查询用户定义的事件，若事件为真，系统将自动恢复/调用与其绑定的任务/钩子并自动重复定时（如果您已开启定时器的自动重装载）。系统初始化后，所有定时查询定时器的值均为零，相当于已经溢出，系统已经开始查询用户定义的事件了。<br>
 **3. 任务与钩子的区别**  由于定时中断钩子、定时查询钩子都是在SysTick_Handler中被调用，所以只有极为精简的代码才适合创建钩子，耗时长的代码应创建为任务。再有，钩子相当于比任务具有更高的优先级，可更及时的被执行。

 **四、一般用户任务** <br>
对于一般用户任务，只要优先级为0级，就是空闲任务。用户空闲任务内都要有阻塞事件，以保证所有空闲任务都有机会被运行。

最高优先级固定分配给任务管理器和调试任务，用户任务不要使用。

### 任务状态
CosyOS的任务状态较传统RTOS划分更为详细，以便用户在使用任务管理器监控任务时，能够更加准确的判断各任务的运行情况。<br>
具体划分如下：<br>

注：[]为在任务管理器中显示的任务状态
*  **运行状态[RDY]**  任务正在运行。
*  **就绪状态[RDY]**  任务已经准备就绪，可以参与任务调度并运行。
*  **延时阻塞状态[DLY]**  由延时而导致任务进入阻塞状态，直到延时时间结束转为就绪状态。
*  **非延时阻塞/超时阻塞状态** ：<br>
    **1. 二值信号量阻塞状态[BIN]**  由二值信号量等待或获取而导致任务进入阻塞状态，直到成功或超时转为就绪状态。<br>
    **2. 互斥信号量阻塞状态[MUT]**  由互斥信号量获取而导致任务进入阻塞状态，直到成功或超时转为就绪状态。<br>
    **3. 计数信号量阻塞状态[SEM]**  由计数信号量获取而导致任务进入阻塞状态，直到成功或超时转为就绪状态。<br>
    **4. 等待标志组阻塞状态[GRP]**  由事件标志组等待而导致任务进入阻塞状态，直到成功或超时转为就绪状态。<br>
    **5. 接收飞信阻塞状态[FET]**  由接收飞信而导致任务进入阻塞状态，直到收到飞信或超时转为就绪状态。<br>
    **6. 接收私信阻塞状态[DM]**   由接收私信而导致任务进入阻塞状态，直到收到私信或超时转为就绪状态。<br>
    **7. 接收邮件阻塞状态[MAL]**  由接收邮件而导致任务进入阻塞状态，直到收到邮件或超时转为就绪状态。<br>
    **8. 接收消息阻塞状态[MSG]**  由接收消息而导致任务进入阻塞状态，直到收到消息或超时转为就绪状态。<br>
*  **浮动状态[RDY]** 
浮动状态是临时的就绪状态，在特定条件下可再次转为刚才的阻塞状态。当任务进入非延时阻塞状态后，终将转为浮动状态。在任务管理器中显示为就绪状态。
*  **超时状态[OSR]** 
当任务累计运行时间超过安全运行时，仍不能被比它优先级低的任务所切换，则进入超时状态并放弃处理器的使用权，直到系统空闲任务运行时才能恢复。
*  **挂起状态[SPD]** 
任务暂停运行，除任务状态变为挂起外，任务的其它相关信息和数据均保持不变。当恢复任务后，可继续挂起时的断点运行。
运行状态的任务被挂起，恢复后为就绪状态；其它状态的任务被挂起，恢复后为原状态。
*  **停止状态**  任务停止运行，不可恢复。<br>
    **1. 任务栈溢出停止[!OF]**  由任务栈溢出而导致任务停止运行。<br>
    **2. 任务栈重分配失败停止[!RF]**  由任务栈重分配失败而导致任务停止运行。<br>
*  **删除/未启动状态（Deleted）** 任务被删除或未启动，若想再次运行该任务，必须重新启动该任务。<br>

![任务状态](images/task_state.png "任务状态")<br>
—————————— CosyOS-任务状态图 ——————————

### 线程优先级
![线程优先级](images/thread_priority.png "线程优先级")<br>

### 中断优先级
无论您的MCU是哪种内核，CosyOS都会占用两个中断做为系统中断。<br>
1.  SysTick_Handler/定时器0中断：优先级固定为最低级，主要功能：软件RTC的计时、软件定时器的计数、调用滴答钩子、恢复/调用定时中断任务/钩子和定时查询任务/钩子。<br>
2.  PendSV_Handler/其它替代中断：优先级固定为最低级，主要功能：中断异步服务的执行、任务异步服务的执行、任务调度与切换。<br>

 _CosyOS的全局不关中断，是全局不关总中断和用户中断。在进入任务临界区时，对于8051、80251内核会关闭上述两个系统中断，对于Arm内核会屏蔽最低优先级的中断。所以，**只要您的用户中断不是最低优先级就可实现零中断延迟**。_ 

注意事项：<br>
1.  对于8051、80251内核，您的用户中断应尽量避免使用最低优先级；对于Arm内核，您的用户中断干脆不要使用最低优先级。<br>
2.  您的用户中断应遵循快进快出的原则，仅执行最为紧急的工作，而把不太紧急的工作转为在任务中执行（中断服务任务）。<br>
3.  您的用户中断的最大执行时间应远小于系统滴答周期，这将促使整个系统更加良性的运行。

补充说明：<br>

其实，CosyOS也可以采用像Keil RTX一样的方法，在任务中调用服务时通过调用SVC_Handler/其它替代中断来执行服务，这样就不用进入任务临界区（不会关闭上述两个系统中断），但这种方法并不适用于CosyOS，原因有三：
1.  调用SVC的方法相较于进入临界区的方法无疑会花更多的时间，对于一些比较老的、主频低的内核来说，这并不是一个好主意。<br>
2.  调用SVC的方法其实还有一个重要目的，就是实现服务的空间隔离，而对于没有Thread模式/Handler模式的内核来说，这种方法是无法实现栈空间隔离的。<br>
3.  调用SVC的方法，SVC的优先级必须比SysTick和PendSV高一级。而对于某些内核来说，中断优先级是非常有限的（如增强的8051共有四级），SysTick+PendSV+SVC就占用了两级，用户中断中要想实现零中断延迟，就只剩两级优先级可用了，而这往往是不够用的。<br>

 _综上所述，对于CosyOS来说，在任务中调用服务时，进入任务临界区的方法优于调用SVC的方法，性能上也会更进一步。_ <br>

之所以补充说明这一点，是因为有人可能会说：“你还不是关中断了吗，关了SysTick和PendSV”？CosyOS虽然是关了中断，但关的是系统专用中断，并没有关总中断和用户中断啊！再者，即使采用了调用SVC的方法，当调用SVC时，如果SysTick或PendSV也同时到来，虽然它们没有被关闭，但也只能乖乖挂起，在后面排队，这又与关闭它们有什么区别呢？<br>

 _可见，无论是调用SVC的方法还是进入任务临界区的方法，对于任务中调用服务结果都是一样的，对零中断延迟都无影响。_ <br>

最重要的事情说三遍：<br>
**★★★ 只要您的用户中断不是最低优先级就可实现零中断延迟 ★★★** <br>
**★★★ 只要您的用户中断不是最低优先级就可实现零中断延迟 ★★★** <br>
**★★★ 只要您的用户中断不是最低优先级就可实现零中断延迟 ★★★** <br>

相关资料：<br>
[全局不关中断原理](https://gitee.com/cosyos/cosyos/blob/master/CosyOS-%E6%89%80%E6%9C%89%E5%86%85%E6%A0%B8%E5%85%A8%E5%B1%80%E4%B8%8D%E5%85%B3%E4%B8%AD%E6%96%AD%E5%8E%9F%E7%90%86.md)

### 中断中调用服务注意事项
1.  发送邮件（当互斥访问机制为可重入访问时）<br>
同一处调用的相邻两次调用，间隔时间不可过短（大于一个滴答周期为宜），以确保局部邮箱不会重入。<br>
实际上只有足够间隔时间时，才适合使用邮箱，否则可能会导致邮件覆盖。<br>
2.  全局变量写访问<br>
同一处调用的相邻两次调用，间隔时间不可过短（大于一个滴答周期为宜），或是局部变量不要发生变化，以确保局部变量不会重入或可重入。<br>
3.  计数信号量释放（8051/251内核）<br>
同一处调用的相邻两次调用，间隔时间不可过短（大于一个滴答周期为宜），否则可能会导致漏记次数。<br>
4.  中断异步服务总数（8051/251内核）<br>
由于8051/251内核，在PendSV中要依次查询所有的中断服务标志，所以，中断异步服务总数不可过多（至多64个），否则有可能造成PendSV的运行时间过长，对系统实时性造成不利影响。建议在中断中仅执行最为紧急的工作，且调用必要的异步服务。不过您也不用过于担心，中断服务标志是bit型变量，查询采用JNB指令（2个机器周期），清零采用CLR指令（1个机器周期），还是非常高效的，仅需注意减少不必要的调用即可。<br>

### 临界区
临界区分为“任务临界区”与“全局临界区”。<br>
 **任务临界区** 任务级的临界区保护，仅关闭任务调度（SysTick、PendSV）；<br>
 **全局临界区** 系统级的临界区保护，会关闭总中断。CosyOS内核中从来不会进入全局临界区，提供此项服务只是为了便于用户对全局公共资源的保护。<br>
两种临界区都支持嵌套功能，可在系统配置头文件中设置嵌套功能的开启或关闭，默认为开启。<br>
![临界区设置](images/ljqqt.png "临界区设置")<br>

### 任务管理器
在您配置好相关硬件和软件后，任务管理器便可以使用了。您可使用某一款串口调试助手，作为任务管理器的输出窗口。
![任务管理器](images/taskmgr_0.png "任务管理器")<br>
—————————— CosyOS-任务管理器 ——————————

![任务管理器](images/taskmgr_1.png "任务管理器")<br>
—————————— CosyOS-任务管理器 ——————————

相关名词解释<br>
**Name**：任务名称<br>
**TID**：任务ID<br>
**TPL**：任务优先级<br>
**STA**：任务状态<br>
**CPU**：CPU使用率<br>
**RAM**：任务栈大小（字节数）<br>

其中，RAM项需着重说明一下。<br>
示例1：32B/m49B，系统采用假定入栈法计算出的可能的最大任务栈占用为32Bytes，任务栈已malloc分配了49Bytes。<br>
示例2：32B/r32B，系统采用假定入栈法计算出的可能的最大任务栈占用为32Bytes，任务栈已realloc分配了32Bytes。<br>
示例3：32B/s32B，系统采用假定入栈法计算出的可能的最大任务栈占用为32Bytes，任务栈已静态创建了32Bytes。<br>

**ISS**：中断服务栈的深度，针对ARM内核，如果您已开启服务栈监控，会显示此项，历史最大值/设置值。<br>

**Task-PC**：任务PC监控，如果您已开启任务PC监控，会显示此项。<br>
当任务管理器被调度运行时，上一个被切换的任务入栈的PC值。当有任务出现代码运行卡死或死循环时，此功能可用来帮助用户锁定具体位置，以便查找BUG。注意，PC值为下一条汇编指令的地址。<br>
某任务出现代码运行卡死或死循环的表现：任务始终为就绪状态，且占用较高的CPU使用率，比它优先级低的任务一直都没有机会运行。

**SysTick**：系统滴答时间统计，如果您已开启系统滴答时间统计，会显示此项。<br>
注意：系统滴答时间是SysTick_Handler运行的平均时间，并非是任务切换时间。

**Alarm**：报警信息，如果有报警，会显示此项。<br>
 :x: rts：任务栈重分配，虽然成功了，但仍要提醒用户任务栈定义小了； :x: <br>
注：上述报警从V3.3.1版本开始已取消，因为任务栈重分配机制被定义为一种常规功能。<br>

otq：任务队列溢出；<br>
otp：任务优先级溢出。<br>
omq：消息队列溢出。<br>
osr：任务运行时间超出安全运行时。<br>

**Fault**：故障信息，如果有故障，会显示此项。<br>
mmn：消息节点内存分配失败；<br>
mtn：任务节点内存分配失败；<br>
mts：任务栈内存分配失败；<br>
rts：任务栈内存重分配失败；<br>
ots：任务栈溢出；<br>
fst：启动任务失败；<br>
erm：中断中接收消息错误（中断中不可接收来自动态消息队列的消息）。<br>
ois：中断服务栈溢出（仅ARM）。<br>

任务管理器现支持如下命令：<br>
1.  taskmgr，启动任务管理器。<br>
2.  exit，退出任务管理器。<br>
3.  taskmgr /s=...，任务管理器更新速度，取值范围：[50~5000]ms。<br>
注意：命令仅支持小写字母，结尾必须加回车换行（\r\n）。

### 私信
私信是CosyOS独创的一种任务间通讯方式，可用来实现信号、事件、消息等功能。<br>
私信的本质其实就是任务线程的形参，功能上相当于消息邮箱。它的主要优势在于易用性，私信参数（数量、名称、类型）可随意定义，较传统的消息邮箱应用更为便利。缺点是私信只能发给任务，中断中不能发送私信。<br>

#### 私信报警
当您使用私信时，编译后会有报警提示。具体表现为，您定义的每个私信参数都对应一个未引用的局部变量（参数），名称为私信参数的名称后面加_。如下方示例，某个任务创建了私信，共有三个参数p、a、b，则相应的会有报警p_、a_、b_，三个未引用的局部变量（参数），但这并不会影响私信功能的正常使用，用户可不必理会或在C51、C251、C/C++标签页屏蔽掉相应的报警。<br>
![输入图片说明](images/BCSM.3.1.png)<br>
8051内核，报警为warning C280；<br>
80251内核，报警为warning C47；<br>
Cortex-M内核，报警为warning： #177-D。<br>

当私信的创建模式为智能创建模式时：<br>
8051内核还另有报警R3_、R5_、R7_，三个未引用的局部变量；<br>
80251内核还另有报警DR0_、DR4_、R11_，三个未引用的参数。
